/*

An example of license ...

This source file is part of Scol
For the latest info, see http://www.scolring.org

Copyright (c) 2010 Iri <iri@irizone.net>

This source is under the terms of the Scol License. See the COPYING included file
for more informations.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.

For others informations, please contact us from http://www.scolring.org/

*/

#include "main.h"

/* définition de deux variables globales établissant la liaison avec les structures partagées du noyau
 la structure cbmachine n'est définie que dans la version MS Windows */
#if defined _WIN32 || defined __WIN32__
cbmachine ww;
#endif
mmachine  mm;

/*
    Fonction interne codant la fonction de l'API Scol "example_increment" tel que définit plas bas,
    lors du chargement de la bibliothèque par le noyau.
    Le but de cette fonction est de simplement incrémenter de 1 la valeur passée en argument de
    la fonction Scol */
int sc_increment (mmachine m)
{
    int mnumber;

    /* fonction de débogage : écrit un message dans le log, dépendant du masque choisi lors de l'exécution.
    Les masques possibles sont :
    (aucun)     -> 0
    MSKFOO      -> 1
    MSKRUNTIME  -> 2
    MSKWARNING  -> 4
    MSKTRACE    -> 8
    MSKDEBUG    -> 16 */
    MMechostr (MSKDEBUG, "sc_increment: entering\n");

    /* On récupère dans la pile les arguments passés à la fonction Scol dans le script.
    L'ordre est inversé : le dernier élément de la pile est le dernier argument de la fonction.
    C'est ce que fait la fonction MMpull : l'élément est donc en même temps supprimé de la pile.

    La macro MTOI permet de convertir l'élément en entier directement utilisable. Cette macro n'est
    utilisable qu'avec des entiers. Pour les autres types, utilisez, si nécessaire, la macro MTOP qui
    vous convertira la valeur retournée par MMpull en un pointeur directement utilisable. */
    mnumber = MTOI (MMpull (m));

    mnumber++;

    /* On ajoute un élément en dernière position à la pile : le nombre incrémenté de 1, grâce à la
    fonction MMpush. la macro ITOM, inverse en quelque sorte de MTOI, donne une valeur correcte à la pile.
    Pour les autres types que int, si nécessaire, utilisez la macro PTOM.
    Dans le script Scol, la fonction retourne donc la valeur trouvée au dernier élément de la pile. */
    MMpush (m, ITOM (mnumber));

    /* Le retour de la fonction est purement interne et ne joue aucun rôle dans la valeur retournée
    par la fonction Scol dans le script Scol. */
    return 0;
}





/*
    Définition de l'API de la bibliothèque.

    Quatre tableaux qui sont présentés un peu plus bas, à la fonction 'SCOLinitEXAMPLEclass'
    Les éléments définis ici seront alors accessibles depuis n'importe quel script Scol.
*/

char* example_name[EXAMPLE_PKG_NB] =
{
    "example_increment"     /* fonction de l'API appelable depuis l'extérieur */
};

int (*example_fun[EXAMPLE_PKG_NB]) (mmachine m) =
{
    sc_increment            /* nom interne où le code de cette fonction est développé */
};

int example_narg[EXAMPLE_PKG_NB] =
{
    1                       /* nombre d'argument attendu par cette fonction */
};

char* example_type[EXAMPLE_PKG_NB] =
{
    "fun [I] I"             /* prototype Scol de cette fonction : on notera que le nombre d'argument est bien 1 */
};


/*
    Fonction défininssant l'API Scol de la bibliothèque.

    Cette API est définie par 4 tableaux de EXAMPLE_PKG_NB cases précisément.
    Il est impératif que les 4 tableaux aient le même nombre de cases et que cenombre soit strictement
    équivalent au nombre de fonctions de l'API.

    Le comppilateur pourra ne rien dire si le nombre de cases est plus grand que le nombre réelle de
    fonctions de l'API (le cas inverse engendrera un warning si vous avez suivi mes options de compilation).
    Cependant, à l'exécution, la bibliothèque ne sera pas chargée et il n'y aura aucun message d'erreur.
    Ŝoyez attentif à ce niveau (comme au reste !).
    Notez que "nombre de fonctions" inclue les fonctions proprement dit mais aussi les objets Scol
    (les types Scol), les flags, etc ... définis dans votre API.

    La fonction interne 'PKhardpak' les chargera dans l'environnement minimal. Ses arguments sont
    respectivement définis comme suit :
    - la machine Scol correspondante
    - le nom interne du package
    - le nombre d'éléments de l'API
    - les noms des éléments de votre API (fonctions, types, flags, ...)
    - leurs correspondants internes
    - le nombre d'argument qu'ils attendent (si cela a un sens, sinon ce sera NULL)
    - le typage Scol

    Les tableaux doivent correspondre, c'est-à-dire que le nom de l'élément de l'API situé à la
    première case doit correspondre la fonction interne situé à la même case du second tableau, qui
    doit correspondre au nombre d'argument situé à la même case du troisième tableau et même chose
    pour le quatrième tableau.

    Enfin, il va de soi que le nombre d'arguments défini pour un élément de l'API dans le troisème
    tableau doit être strictement cohérent avec le typage défini au quatrième tableau.

    Par exemple : si le nombre d'argument est 2, le typage de la fonction pourra être "fun [I S] I"
    ou "fun [Chn S]" mais pas "fun [I S I]" ou "fun [S] I".
    Là encore, aucune erreur ne peut être détecté lors de la compilation mais lorsque la tentative de
    chargement de votre bibliothèque par le noyau échouera immanquablement sans la moindre erreur
    apparente.
*/
int SCOLinitEXAMPLEclass (mmachine m)
{
    int k = 0;

    MMechostr (MSKDEBUG, "SCOLinitEXAMPLEclass library : loading\n");

    k = PKhardpak (m, "EXAMPLEengine", EXAMPLE_PKG_NB, example_name, example_fun, example_narg, example_type);
    return k;
}


/*
    Fonction de chargement de la bibliothèque par le noyau.

    C'est donc cette fonction qui sera appelé par le noyau pour charger cette bibliothèque dans l'environnement
    minimal de chaque VM Scol exécutée.
    Le nom de cette fonction, ici 'SCOLloadEXAMPLE', devra être reportée dans la ligne adéquate du fichier usm.ini :

    # sous MS Windows :
    plugin plugins/lib.example.dll SCOLloadEXAMPLE SCOLfreeEXAMPLE
    # sous GNU / Linux :
    plugin plugins/lib.example.so SCOLloadEXAMPLE SCOLfreeEXAMPLE

    Cette fonction contient la fonction contenant API Scol ('SCOLinitEXAMPLEclass'), utilisable par n'importe quel
    script Scol (du moment que cettelibrairie est chargée dans l'environnement minimal tel que décrit ci-dessus).

    Enfin, il est à noter que le chargement est différent sous MS Windows et GNU / Linux. On utilise donc le
    preprocesseur pour aiguiller le bon chargement selon la plateforme utilisée lors de la compilation.
*/

/* Chargement pour MS Windows */
#if defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLloadEXAMPLE (mmachine m, cbmachine w)
{
    int k = 0;

    /* Affectation aux variables globales ww et mm des valeurs envoyées par le noyau des structures partagées.
    Il est impératif de les ajouter AVANT tout appel à des fonctions ou macros Scol sous peine de crash immédiat
    lors de l'exécution de Scol. */
    ww = w;
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}

/* Chargement pour GNU / Linux */
#elif defined linux || defined __linux
int SCOLloadEXAMPLE (mmachine m)
{
    int k = 0;

    /* Affectation à la variable globale mm des valeurs envoyées par le noyau des structures partagées.
    Il est impératif de l'ajouter AVANT tout appel à des fonctions ou macros Scol sous peine de crash immédiat
    lors de l'exécution de Scol. */
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}

/* Autres plateformes (le compilateur retournera l'erreur et s'arrêtera) */
#else
#error no platform supported
#endif

/*
    Fonction de déchargement / libération

    La fonction est appelée lorsque Scol se termine. Elle définit, comme pour la fonction de chargement,
    dans le fichier usm.ini (voir ci-dessus).

    Les éventuels objets Scol présents devront avoir été détruits au préalable
*/

#if defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLfreeEXAMPLE ()
#elif defined linux || defined __linux
int SCOLfreeEXAMPLE ()
#else
#error no platform supported
#endif
{
    MMechostr(MSKDEBUG, "\nEXAMPLE library release !\n");
    return 0;
}
